package com.pseudocode.photolib.transforms;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;

/**
 * Reusable transformation definition
 */
@JsonIgnoreProperties(ignoreUnknown=true)
public class ImageTransform {

	private String description;
	private boolean crop;
	private double horzAlign;
	private double vertAlign;
	private int maxHeight;
	private int maxWidth;
	private ImageOrientation orientation;
	private boolean oversize;
	private int blur;
	private int quality;
	private ImageFormat imageFormat;

	/**
	 * Gets a description (only really used as a comment in JSON)
	 */
	@JsonProperty
	public String description() {
		return description;
	}
	
	/**
	 * Sets a description (only really used as a comment in JSON)
	 */
	@JsonProperty
	public ImageTransform description(String value) {
		description = value;
		return this;
	}

	/**
	 * Gets if should crop to fit shape
	 */
	@JsonProperty
	public boolean crop() {
		return crop;
	}
	
	/**
	 * Sets if should crop to fit shape
	 */
	@JsonProperty
	public ImageTransform crop(boolean value) {
		crop = value;
		return this;
	}

	/**
	 * Gets the horizontal alignment of an image when cropped
	 */
	@JsonProperty
	public double horzAlign() {
		return horzAlign;
	}

	/**
	 * Sets the horizontal alignment of an image when cropped
	 */
	@JsonProperty
	public ImageTransform horzAlign(double value) {
		horzAlign = value;
		return this;
	}

	/**
	 * Gets the vertical alignment of an image when cropped
	 */
	@JsonProperty
	public double vertAlign() {
		return vertAlign;
	}

	/**
	 * Sets the vertical alignment of an image when cropped
	 */
	@JsonProperty
	public ImageTransform vertAlign(double value) {
		vertAlign = value;
		return this;
	}

	/**
	 * Gets the max-height of the resulting image
	 */
	@JsonProperty
	public int maxHeight() {
		return maxHeight;
	}

	/**
	 * Sets the max-height of the resulting image
	 */
	@JsonProperty
	public ImageTransform maxHeight(int value) {
		maxHeight = value;
		return this;
	}

	/**
	 * Gets the max-width of the resulting image
	 */
	@JsonProperty
	public int maxWidth() {
		return maxWidth;
	}

	/**
	 * Sets the max-width of the resulting image
	 */
	@JsonProperty
	public ImageTransform maxWidth(int value) {
		maxWidth = value;
		return this;
	}

	/**
	 * Gets the transform orientation
	 */
	@JsonProperty
	public ImageOrientation orientation() {
		return orientation;
	}

	/**
	 * Sets the transform orientation
	 */
	@JsonProperty
	public ImageTransform orientation(ImageOrientation value) {
		orientation = value;
		return this;
	}

	/**
	 * Gets if should allow scaling up small images
	 */
	@JsonProperty
	public boolean oversize() {
		return oversize;
	}

	/**
	 * Sets if should allow scaling up small images
	 */
	@JsonProperty
	public ImageTransform oversize(boolean value) {
		oversize = value;
		return this;
	}

	/**
	 * Gets the Gaussian blur radius
	 */
	@JsonProperty
	public int blur() {
		return blur;
	}

	/**
	 * Sets the Gaussian blur radius
	 */
	@JsonProperty
	public ImageTransform blur(int value) {
		blur = value;
		return this;
	}

	/**
	 * Gets the quality level 0-100 (JPEG only)
	 */
	@JsonProperty
	public int quality() {
		return quality;
	}

	/**
	 * Sets the quality level 0-100 (JPEG only)
	 */
	@JsonProperty
	public ImageTransform quality(int value) {
		quality = value;
		return this;
	}

	/**
	 * Gets the target image format
	 */
	@JsonProperty
	public ImageFormat imageFormat() {
		return imageFormat;
	}

	/**
	 * Sets the target image format
	 */
	@JsonProperty
	public ImageTransform imageFormat(ImageFormat value) {
		imageFormat = value;
		return this;
	}

	/**
	 * Validates that the transform properties contain valid values.
	 */
	void validate() {
		if (this.crop) {
			// ensure only predefined alignments
			if (this.horzAlign < 0.0 || this.horzAlign > 1.0) {
				throw new IllegalArgumentException("horzAlign");
			}
			if (this.vertAlign < 0.0 || this.vertAlign > 1.0) {
				throw new IllegalArgumentException("vertAlign");
			}
		}

		if (this.maxWidth < 0) {
			// NOTE: zero indicates no width constraint
			throw new IllegalArgumentException("maxWidth");
		}

		if (this.maxHeight < 0) {
			// NOTE: zero indicates no height constraint
			throw new IllegalArgumentException("maxHeight");
		}

		if (this.quality < 0 || this.quality > 100) {
			throw new IllegalArgumentException("quality");
		}

		// NOTE: if target imageFormat is null, it uses the original imageFormat
	}

	StringBuilder toString(StringBuilder builder) {

		if (builder == null) {
			throw new NullPointerException("builder");
		}
		
		this.validate();

		final char FEATURE_SEPARATOR_CHAR = '-';
		final char DIMENSION_SEPARATOR_CHAR = 'x';
		final char OVERSIZE_INDICATOR = 'F';
		final char ORIENTATION_PREFIX = 'r';
		final char BLUR_PREFIX = 'b';

		// {quality}-{max-width}x{max-height}-{oversize}-{horz-align}x{vert-align}-{orientation}-{blur}.{format}
		builder

			// "{quality}"
			.append(quality)

			//"-{max-width}x{max-height}"
			.append(FEATURE_SEPARATOR_CHAR)
			.append(maxWidth)
			.append(DIMENSION_SEPARATOR_CHAR)
			.append(maxHeight);

		if (oversize) {
			//"-{oversize}"
			builder
				.append(FEATURE_SEPARATOR_CHAR)
				.append(OVERSIZE_INDICATOR);
		}

		if (crop) {
			//"-{horz-align}x{vert-align}"
			builder
				.append(FEATURE_SEPARATOR_CHAR)
				.append(Math.round(1000*horzAlign))
				.append(DIMENSION_SEPARATOR_CHAR)
				.append(Math.round(1000*vertAlign));
		}

		if (orientation != null &&
			orientation != ImageOrientation.INVALID &&
			orientation != ImageOrientation.NORMAL) {
			//-{orientation}"
			builder
				.append(FEATURE_SEPARATOR_CHAR)
				.append(ORIENTATION_PREFIX)
				.append(orientation.ordinal());
		}

		if (blur > 0) {
			//-{blur}"
			builder
				.append(FEATURE_SEPARATOR_CHAR)
				.append(BLUR_PREFIX)
				.append(blur);
		}

		return builder;
	}

	@Override
	public String toString() {
		return toString( new StringBuilder(20) ).toString();
	}
}
